@@ -29,7 +29,6 @@
 #include "util.h"
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnomecanvas/libgnomecanvas.h>
 
 #define GTK_RESPONSE_CREDITS 0
 #define ZENITY_HELP_PATH ZENITY_DATADIR "/help/"
@@ -61,6 +60,21 @@ static const char *documenters[] = {
 
 static gchar *translators;
 
+static const char *license[] = {
+	N_("This program is free software; you can redistribute it and/or modify "
+	   "it under the terms of the GNU Lesser General Public License as published by "
+	   "the Free Software Foundation; either version 2 of the License, or "
+	   "(at your option) any later version.\n"),
+	N_("This program is distributed in the hope that it will be useful, "
+	   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+	   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+	   "GNU Lesser General Public License for more details.\n"),
+	N_("You should have received a copy of the GNU Lesser General Public License "
+	   "along with this program; if not, write to the Free Software "
+	   "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.")
+};
+
+#if 0
 static gint
 zenity_move_clothes_event (GnomeCanvasItem *item, 
                            GdkEvent *event,
@@ -238,16 +252,22 @@ zenity_zen_wisdom (GtkDialog *dialog, GdkEventKey *event, gpointer user_data)
 
   return FALSE;
 }
+#endif
 
 void 
 zenity_about (ZenityData *data)
 {
   GdkPixbuf *logo;
   GtkWidget *help_button;
+  char *license_trans;
+
 
   translators = _("translator-credits");
   logo = gdk_pixbuf_new_from_file (ZENITY_IMAGE_FULLPATH ("zenity.png"), NULL);
 
+  license_trans = g_strconcat (_(license[0]), "\n", _(license[1]), "\n",
+                               _(license[2]), "\n", NULL);
+
   dialog = gtk_about_dialog_new ();
 
   g_object_set (G_OBJECT (dialog),
@@ -258,8 +278,13 @@ zenity_about (ZenityData *data)
 		"authors", authors,
                 "documenters", documenters,
                 "translator-credits", translators,
+                "website", "http://live.gnome.org/Zenity",
                 "logo", logo,
+                "wrap-license", TRUE,
+                "license", license_trans,
 		NULL);
+ 
+  g_free (license_trans);
 
   zenity_util_set_window_icon (dialog, NULL, ZENITY_IMAGE_FULLPATH ("zenity.png"));
 
@@ -277,8 +302,11 @@ zenity_about (ZenityData *data)
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (zenity_about_dialog_response), data);
+
+#if 0
   g_signal_connect (G_OBJECT (dialog), "key_press_event",
                     G_CALLBACK (zenity_zen_wisdom), NULL);
+#endif
 
   zenity_util_show_dialog (dialog);
   gtk_main ();
@@ -292,7 +320,6 @@ zenity_about_dialog_response (GtkWidget *widget, int response, gpointer data)
   switch (response) {
     case GTK_RESPONSE_CLOSE:
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
-      gtk_main_quit ();
       break;
 
     default:
@@ -300,6 +327,8 @@ zenity_about_dialog_response (GtkWidget *widget, int response, gpointer data)
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
       break;
   }
+
+  gtk_main_quit ();
 }
 
 static void
