@@ -23,7 +23,6 @@
 
 #include "config.h"
 
-#include <glade/glade.h>
 #include <string.h>
 #include "zenity.h"
 #include "util.h"
@@ -76,7 +75,10 @@ void zenity_fileselection (ZenityData *data, ZenityFileData *file_data)
 
     if (file_data->uri[strlen (file_data->uri) - 1] != '/') {
       basename = g_path_get_basename (file_data->uri);
-      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), basename);
+      if (file_data->save)
+        gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), basename);
+      else
+        (void) gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), file_data->uri);
       g_free (basename);
     }
     g_free (dir);
