@@ -25,7 +25,6 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <time.h>
 #include <string.h>
 
@@ -48,6 +47,7 @@ zenity_notification_icon_update (void)
   GError *error = NULL;
 
   pixbuf = gdk_pixbuf_new_from_file_at_scale (icon_file, icon_size, icon_size, TRUE, &error);
+
   if (error) {
     g_warning ("Could not load notification icon '%s': %s",
                icon_file, error->message);
@@ -165,10 +165,18 @@ zenity_notification_handle_stdin (GIOChannel *channel,
           NotifyNotification *notif;
           const gchar *icon = NULL;
           gchar *freeme = NULL;
-          gchar *message;
+          gchar **message;
           error = NULL;
 
-          message = g_strcompress (value);
+          /* message[1] (the summary) will be NULL in case there's
+           * no \n in the string. In which case only the title is
+           * defined */
+          message = g_strsplit (g_strcompress (value), "\n", 2);
+
+          if (*message == NULL) {
+            g_printerr (_("Could not parse message from stdin\n"));
+            continue;
+          }
 
           if (icon_stock) {
             icon = icon_stock;
@@ -176,9 +184,12 @@ zenity_notification_handle_stdin (GIOChannel *channel,
             icon = freeme = g_filename_to_uri (icon_file, NULL, NULL);
           }
 
-          notif = notify_notification_new_with_status_icon (message, NULL /* summary */,
-                					    icon, status_icon);
-          g_free (message);
+          notif = notify_notification_new_with_status_icon (
+                              message[0] /* title */,
+                              message[1] /* summary */,
+                              icon, status_icon);
+
+          g_strfreev (message);
           g_free (freeme);
 
 	  notify_notification_show (notif, &error);
@@ -197,7 +208,7 @@ zenity_notification_handle_stdin (GIOChannel *channel,
 	}
       } else if (!g_ascii_strcasecmp (command, "tooltip")) {
         if (g_utf8_validate (value, -1, NULL)) {
-          gtk_status_icon_set_tooltip (status_icon, value);
+          gtk_status_icon_set_tooltip_text (status_icon, value);
         } else {
           g_warning ("Invalid UTF-8 in input!");
         }
@@ -246,9 +257,9 @@ zenity_notification (ZenityData *data, ZenityNotificationData *notification_data
 		    G_CALLBACK (zenity_notification_icon_size_changed_cb), data);
 
   if (notification_data->notification_text) {
-    gtk_status_icon_set_tooltip (status_icon, notification_data->notification_text);
+    gtk_status_icon_set_tooltip_text (status_icon, notification_data->notification_text);
   } else {
-    gtk_status_icon_set_tooltip (status_icon, _("Zenity notification"));
+    gtk_status_icon_set_tooltip_text (status_icon, _("Zenity notification"));
   }
 
   icon_file = g_strdup (data->window_icon);
@@ -279,6 +290,11 @@ zenity_notification (ZenityData *data, ZenityNotificationData *notification_data
 
   /* Show icon and wait */
   gtk_status_icon_set_visible (status_icon, TRUE);
+
+  if(data->timeout_delay > 0) {
+    g_timeout_add (data->timeout_delay * 1000, (GSourceFunc) zenity_util_timeout_handle, NULL);
+  }
+
   gtk_main ();
 
   /* Cleanup */
