@@ -29,14 +29,14 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <signal.h>
-#include <glade/glade.h>
 #include "zenity.h"
 #include "util.h"
 
-static GladeXML *glade_dialog;
+static GtkBuilder *builder;
 static ZenityData *zen_data;
 static GIOChannel *channel;
 
+static gint pulsate_timeout = -1;
 static gboolean autokill;
 
 gint zenity_progress_timeout (gpointer data);
@@ -51,20 +51,38 @@ zenity_progress_pulsate_progress_bar (gpointer user_data)
   return TRUE;
 }
 
+static void
+zenity_progress_pulsate_stop ()
+{
+  if (pulsate_timeout > 0) {
+    g_source_remove (pulsate_timeout);
+    pulsate_timeout = -1;
+  }
+}
+
+static void
+zenity_progress_pulsate_start (GObject *progress_bar)
+{
+  if (pulsate_timeout == -1) {
+    pulsate_timeout = g_timeout_add (100,
+                                     zenity_progress_pulsate_progress_bar,
+                                     progress_bar);
+  }
+}
+
 static gboolean
 zenity_progress_handle_stdin (GIOChannel   *channel,
                               GIOCondition  condition,
                               gpointer      data)
 {
   static ZenityProgressData *progress_data;
-  static GtkWidget *progress_bar;
-  static GtkWidget *progress_label;
-  static gint pulsate_timeout = -1;
+  static GObject *progress_bar;
+  static GObject *progress_label;
   float percentage = 0.0;
   
   progress_data = (ZenityProgressData *) data;
-  progress_bar = glade_xml_get_widget (glade_dialog, "zenity_progress_bar");
-  progress_label = glade_xml_get_widget (glade_dialog, "zenity_progress_text");
+  progress_bar = gtk_builder_get_object (builder, "zenity_progress_bar");
+  progress_label = gtk_builder_get_object (builder, "zenity_progress_text");
 
   if ((condition == G_IO_IN) || (condition == G_IO_IN + G_IO_HUP)) {
     GString *string;
@@ -73,8 +91,7 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
     string = g_string_new (NULL);
 
     if (progress_data->pulsate) {
-      if (pulsate_timeout == -1)
-        pulsate_timeout = g_timeout_add (100, zenity_progress_pulsate_progress_bar, progress_bar);
+      zenity_progress_pulsate_start (progress_bar);
     }
 
     while (channel->is_readable != TRUE)
@@ -106,26 +123,58 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
         match = g_strstr_len (string->str, strlen (string->str), "#");
         match++;
         gtk_label_set_text (GTK_LABEL (progress_label), g_strcompress(g_strchomp (g_strchug (match))));
+
+      } else if (g_str_has_prefix (string->str, "pulsate")) {
+        gchar *colon, *command, *value;
+
+        zenity_util_strip_newline (string->str);
+
+        colon = strchr(string->str, ':');
+        if (colon == NULL) {
+            continue;
+        }
+
+        /* split off the command and value */
+        command = g_strstrip (g_strndup (string->str, colon - string->str));
+
+        value = colon + 1;
+        while (*value && g_ascii_isspace (*value)) value++;
+
+        if (!g_ascii_strcasecmp (value, "false")) {
+          zenity_progress_pulsate_stop ();
+
+          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
+                                         progress_data->percentage / 100.0);
+        } else {
+          zenity_progress_pulsate_start (progress_bar);
+        }
+
+        g_free (command);
       } else {
 
         if (!g_ascii_isdigit (*(string->str)))
           continue;
 
         /* Now try to convert the thing to a number */
-        percentage = atoi (string->str);
-        if (percentage >= 100) {
-	  GtkWidget *button;
-	  button = glade_xml_get_widget( glade_dialog,"zenity_progress_ok_button");
-          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 1.0);
-	  gtk_widget_set_sensitive(GTK_WIDGET (button), TRUE);
-	  gtk_widget_grab_focus(GTK_WIDGET (button));
-	  if (progress_data->autoclose) {
-		zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
-		gtk_main_quit();
-		
-	  }
-	} else
-          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), percentage / 100.0);
+        percentage = CLAMP(atoi (string->str), 0, 100);
+
+        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
+                                       percentage / 100.0);
+
+        progress_data->percentage = percentage;
+
+        if (percentage == 100) {
+          GObject *button;
+
+          button = gtk_builder_get_object(builder, "zenity_progress_ok_button");
+          gtk_widget_set_sensitive(GTK_WIDGET (button), TRUE);
+          gtk_widget_grab_focus(GTK_WIDGET (button));
+
+          if (progress_data->autoclose) {
+            zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
+            gtk_main_quit();
+          }
+        }
       }
 
     } while (g_io_channel_get_buffer_condition (channel) == G_IO_IN);
@@ -136,28 +185,27 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
     /* We assume that we are done, so stop the pulsating and de-sensitize the buttons */
     GtkWidget *button;
 
-    button = glade_xml_get_widget (glade_dialog, "zenity_progress_ok_button");
+    button = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                 "zenity_progress_ok_button"));
     gtk_widget_set_sensitive (button, TRUE);
     gtk_widget_grab_focus (button);
 
-    button = glade_xml_get_widget (glade_dialog, "zenity_progress_cancel_button");
+    button = GTK_WIDGET (gtk_builder_get_object (builder,
+                         "zenity_progress_cancel_button"));
+
     gtk_widget_set_sensitive (button, FALSE);
-		
+
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 1.0);
 
-    if (progress_data->pulsate) {
-      g_source_remove (pulsate_timeout);
-      pulsate_timeout = -1;
-    }
+    zenity_progress_pulsate_stop ();
 
-    if (glade_dialog)
-      g_object_unref (glade_dialog);
+    g_object_unref (builder);
 
     if (progress_data->autoclose) {
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
       gtk_main_quit();
     }
-		
+
     g_io_channel_shutdown (channel, TRUE, NULL);
     return FALSE;
   }
@@ -177,24 +225,25 @@ void
 zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 {
   GtkWidget *dialog;
-  GtkWidget *text;
-  GtkWidget *progress_bar;
+  GObject *text;
+  GObject *progress_bar;
 
   zen_data = data;
-  glade_dialog = zenity_util_load_glade_file ("zenity_progress_dialog");
+  builder = zenity_util_load_ui_file ("zenity_progress_dialog", NULL);
 
-  if (glade_dialog == NULL) {
+  if (builder == NULL) {
     data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
     return;
   }
-	
-  glade_xml_signal_autoconnect (glade_dialog);
-	
-  dialog = glade_xml_get_widget (glade_dialog, "zenity_progress_dialog");
+
+  gtk_builder_connect_signals (builder, NULL);
+
+  dialog = GTK_WIDGET (gtk_builder_get_object (builder,
+                                               "zenity_progress_dialog"));
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (zenity_progress_dialog_response), data);
-	
+
   if (data->dialog_title)
     gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
 
@@ -203,12 +252,12 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
   if (data->width > -1 || data->height > -1)
     gtk_window_set_default_size (GTK_WINDOW (dialog), data->width, data->height);
 
-  text = glade_xml_get_widget (glade_dialog, "zenity_progress_text");
+  text = gtk_builder_get_object (builder, "zenity_progress_text");
 
   if (progress_data->dialog_text)
     gtk_label_set_markup (GTK_LABEL (text), g_strcompress (progress_data->dialog_text));
 
-  progress_bar = glade_xml_get_widget (glade_dialog, "zenity_progress_bar");
+  progress_bar = gtk_builder_get_object (builder, "zenity_progress_bar");
 
   if (progress_data->percentage > -1)
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 
@@ -233,17 +282,15 @@ zenity_progress_dialog_response (GtkWidget *widget, int response, gpointer data)
     case GTK_RESPONSE_OK:
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
       break;
-		
+
     case GTK_RESPONSE_CANCEL:
       /* We do not want to kill the parent process, in order to give the user
          the ability to choose the action to be taken. See bug #310824.
-		 -- Monday 27, March 2006
-		 But we want to give people the option to choose this behavior.
+         But we want to give people the option to choose this behavior.
+         -- Monday 27, March 2006
       */
       if (autokill) {
         kill (getppid (), 1);
-        zen_data->exit_code = zenity_util_return_exit_code (ZENITY_CANCEL);
-        break;
       }
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_CANCEL);
       break;
